#!/usr/bin/env python3
#
# Updater script of CVE/CPE database
#
# Software is free software released under the "GNU Affero General Public License v3.0"
#
# Copyright (c) 2012-2019  Alexandre Dulaunoy - a@foo.be
# Copyright (c) 2014-2018  Pieter-Jan Moreels - pieterjan.moreels@gmail.com

import argparse
import logging
import os
import shlex
import subprocess
import sys
import time

from CveXplore import CveXplore

runPath = os.path.dirname(os.path.realpath(__file__))
sys.path.append(os.path.join(runPath, ".."))

from lib.LogHandler import UpdateHandler
from lib.Sources_process import (
    CPERedisBrowser,
)
from lib.Config import Configuration
from lib.DatabaseLayer import getSize, dropCollection, getTableNames

logging.setLoggerClass(UpdateHandler)

logger = logging.getLogger("DBUpdater")


def nbelement(collection=None):
    if collection is None or collection == "cve":
        collection = "cves"
    return getSize(collection)


def dropcollection(collection=None):
    if collection is None:
        return False
    if collection == "cve":
        collection = "cves"
    ret = dropCollection(collection)
    return ret


def main(args):
    cvex = CveXplore()

    if args.f:
        logger.info("==========================")
        logger.info("Repopulate")
        logger.info(time.strftime("%a %d %B %Y %H:%M", time.gmtime()))
        logger.info("==========================")
        to_drop = ["cpeother", "mgmt_whitelist", "mgmt_blacklist", "info", "schema"]
        for each in to_drop:
            logger.info(f"Dropping metadata: {each}")
            dropcollection(each)
        if args.l:
            logger.info(
                "Drop collections (-f) and running in loop (-l) used together; only dropping on the first iteration."
            )
        logger.info("Starting initial import...")
        cvex.database.initialize()

    loop = True
    loop_count = 0

    while loop:
        if not args.l:
            loop = False
        else:
            loop_count += 1

        logger.info("==========================")
        if args.m:
            redis_info = "; minimal import without redis-cache-cpe source"
        elif args.c:
            redis_info = "; CPE redis cache enabled"
        else:
            redis_info = ""
        if args.l:
            loop_info = f" (loop #{loop_count})"
        else:
            loop_info = ""
        logger.info(f"Update{redis_info}{loop_info}")
        logger.info(time.strftime("%a %d %B %Y %H:%M", time.gmtime()))
        logger.info("==========================")

        if args.c and args.m:
            logger.warning(
                f"CPE cache enabled (-c) does not do anything when "
                f"minimal import without redis-cache-cpe source (-m) is used"
            )

        cvex.database.update()

        newelement = 0
        for source in sources:
            # Drop collections only if this was the first iteration of a repopulation
            if args.f and source["name"] != "redis-cache-cpe" and loop_count < 2:
                logger.info("Repopulation; dropping collection: " + source["name"])
                if dropcollection(collection=source["name"]):
                    logger.info(f"{source['name']} dropped")
                else:
                    logger.info(f"{source['name']} did not exist")

            if (
                not Configuration.includesFeed(source["name"])
                and source["name"] != "redis-cache-cpe"
            ):
                logger.info(f"Skipping non-configured source: {source['name']}")
                continue

            if source["name"] == "cpeother":
                if "cpeother" not in getTableNames():
                    continue
            if source["name"] != "redis-cache-cpe":
                logger.info("Starting " + source["name"])
                before = nbelement(collection=source["name"])

                if isinstance(source["updater"], str):
                    subprocess.Popen((shlex.split(source["updater"]))).wait()
                else:
                    up = source["updater"]()
                    up.update()

                after = nbelement(collection=source["name"])
                message = (
                    source["name"]
                    + " has "
                    + str(after)
                    + " elements ("
                    + str(after - before)
                    + " update)"
                )
                newelement = str(after - before)
                logger.info(message)
            elif args.c is True and source["name"] == "redis-cache-cpe":
                logger.info("Starting " + source["name"])
                up = source["updater"]()
                up.update()
                logger.info(source["name"] + " updated")

        if args.i and int(newelement) > 0:
            logger.info("Indexing new cves entries in the fulltext indexer...")
            subprocess.Popen(
                (
                    shlex.split(
                        "{} {} {}".format(
                            sys.executable,
                            os.path.join(runPath, "db_fulltext.py"),
                            "-v -l " + newelement,
                        )
                    )
                )
            ).wait()

        if args.l:
            logger.info("Sleeping 1 hour...")
            time.sleep(3600)


if __name__ == "__main__":
    sources = [
        {
            "name": "cpeother",
            "updater": "{} {}".format(
                sys.executable, os.path.join(runPath, "db_mgmt_cpe_other_dictionary.py")
            ),
        },
    ]

    argParser = argparse.ArgumentParser(description="Database updater for cve-search")
    argParser.add_argument(
        "-v", action="store_true", help="Dummy option for backwards compatibility"
    )
    argParser.add_argument(
        "-l", action="store_true", help="Running at regular interval", default=False
    )
    argParser.add_argument(
        "-i",
        action="store_true",
        help="Indexing new cves entries in the fulltext indexer",
        default=False,
    )
    argParser.add_argument(
        "-c", action="store_true", help="Enable CPE redis cache", default=False
    )
    argParser.add_argument(
        "-f",
        action="store_true",
        help="Drop collections and force initial import",
        default=False,
    )
    argParser.add_argument(
        "-m",
        action="store_true",
        help="Minimal import without redis-cache-cpe source",
        default=False,
    )

    args = argParser.parse_args()

    if not args.m:
        sources.extend(
            [
                {"name": "redis-cache-cpe", "updater": CPERedisBrowser},
            ]
        )

    main(args)
